/*
********************************************************************************
**
** Name: PictTools.c
**
** Author: Cary Farrier
**
** Description:
**
**	Support routines for PICTs.
**
** Copyright  1996 Cary Farrier.  This file may be reused in any way you wish.
**
********************************************************************************
*/
#include <Types.h>
#include <Resources.h>
#include <QDOffscreen.h>
#include <Quickdraw.h>

#include "DSpSV_PictTools.h"

/*
********************************************************************************
**
** Function: LoadPictIntoGWorld
**
** Description:
**
**	Loads a PICT resource into a GWorld.  The GWorld is created by this
**	function, and will of the bounds specified in the input parameters.
**
********************************************************************************
*/
OSStatus						/* result code							*/
LoadPictIntoGWorld(
	SInt16 inPictID,			/* resource id of 'PICT'				*/
	UInt16 inDepth,				/* depth of created GWorld				*/
	UInt16 inMaxWidth,			/* max width of created GWorld			*/
	UInt16 inMaxHeight,			/* max height of created GWorld			*/
	GWorldPtr *outGWorld		/* the created gworld					*/
)
{
	PicHandle thePicture;
	OSStatus theError = noErr;
	Rect theRect;
	CGrafPtr theSavePort;
	GDHandle theSaveGDevice;
	
	/* check depth input */
	if( inDepth == 0 )
		inDepth = 8;
		
	/* save port & device */
	GetGWorld( &theSavePort, &theSaveGDevice );
	
	/* load the picture */
	thePicture = GetPicture( inPictID );
	if( NULL == thePicture )
		theError = memFullErr;
	else if( ResError() )
		theError = ResError();
	else
		HLock( (Handle)thePicture );
	
	/* allocate the GWorld */
	if( noErr == theError )
	{
		if( inMaxWidth || inMaxHeight )
		{
			SetRect( &theRect, 0, 0, inMaxWidth, inMaxHeight );
		}
		else
		{
			/* move the PICT rect to 0,0 */
			theRect = (**thePicture).picFrame;
			OffsetRect( &theRect, -theRect.left, -theRect.top );
		}

		theError = NewGWorld( outGWorld, inDepth, &theRect, NULL, NULL, 0 );
	}
	
	/* draw the picture */
	if( noErr == theError )
	{
		SetGWorld( *outGWorld, NULL );
		ClipRect( &theRect );
		DrawPicture( thePicture, &theRect );
	}
	
	/* release PICT memory */
	if( thePicture )
		ReleaseResource( (Handle)thePicture );
	
	/* restore port & device */
	SetGWorld( theSavePort, theSaveGDevice );
	
	/* done */
	return theError;
}