// Quickdraw 3D sample code
//
// Nick Thompson, AppleLink: DEVSUPPORT (devsupport@applelink.apple.com)
//
// 1994-5 Apple Computer Inc., All Rights Reserved

// system headers
#include <Devices.h>
#include <Events.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <Fonts.h>
#include <Menus.h>
#include <PictUtils.h>
#include <QDOffScreen.h>
#include <QuickDraw.h>
#include <SegLoad.h>
#include <StandardFile.h>
#include <TextEdit.h>
#include <ToolUtils.h>

// for QuickDraw 3D
#include <QD3D.h>
#include <QD3DMath.h>
#include <QD3DDrawContext.h>
#include <QD3DShader.h>
#include <QD3DTransform.h>
#include <QD3DGroup.h>
#include <QD3DCamera.h>

#include "DSpSV_Shell.h"
#include "DSpSV_QD3D.h"
#include "DSpSV_ErrorHandler.h"
#include "DSpSV_DSpSupport.h"

//#define DEBUG 1
#ifdef DEBUG
	#define DEBUGSTR(p) DebugStr((p))
#else
	#define DEBUGSTR(p)
#endif

// uncomment following line to show the backplane (rotating sprocket logo)
#define SHOW_BACKPLANE 1

//-------------------------------------------------------------------------------------------
// function prototypes

static void 		InitToolbox( void ) ;
static void 		MainEventLoop( void ) ;
static void 		HandleKeyPress(EventRecord *event) ;
static void 		HandleOSEvent(EventRecord *event) ;
void InitDocumentData( DocumentPtr theDocument ) ;
TQ3Status DocumentDraw3DData( DocumentPtr theDocument ) ;
void DisposeDocumentData( DocumentPtr theDocument) ;

//-------------------------------------------------------------------------------------------
//

Boolean 		gQuitFlag 		= false ;
WindowPtr		gMainWindow		= nil ;
DocumentRec		gDocument ;

//-------------------------------------------------------------------------------------------
// main()
// entry point for the application, initialize the toolbox, initialize QuickDraw 3D
// and enter the main event loop.  On exit from the main event loop, we want to call
// the QuickDraw 3D exit function to clean up QuickDraw 3d.

void main(void)
{
	TQ3Status	myStatus;
	Rect		rBounds = { 50, 50, 350, 350 } ;
	Str255		title = "\pSpinning Box" ;
	FSSpec		theFileSpec ;				// the file we are opening
	OSStatus	theError;
	
	InitToolbox() ;

	if(MetafileFileSpecify( &theFileSpec )) {

		theError = DisplayInit();
		if( theError )
		{
			DEBUGSTR("\p DisplayInit failure.");	
			ExitToShell() ;
		}
				
		SetCursor(*(GetCursor(watchCursor)));
		
		//	Initialize QuickDraw 3D, open a connection to the QuickDraw 3D library
		myStatus = Q3Initialize();
		if ( myStatus == kQ3Failure ) {
			DEBUGSTR("\pErInitialize returned failure.");	
			ExitToShell() ;
		}

		theError = DisplayFadeOut();
		if( theError )
		{
			DEBUGSTR("\p DisplayFadeOut failure.");	
			ExitToShell() ;
		}

		theError = DisplayActivate();
		if( theError )
		{
			DEBUGSTR("\p DisplayActivate failure.");	
			ExitToShell() ;
		}

		theError = DisplayFadeIn();
		if( theError )
		{
			DEBUGSTR("\p DisplayFadeIn failure.");	
			ExitToShell() ;
		}

		SetCursor(&qd.arrow) ;

		// install the error handler - this gets called whenever
		// an error occurs, which means we don't have to check so 
		// much
		Q3Error_Register( MyErrorHandler, 0L );		
		Q3Warning_Register( MyWarningHandler, 0L );		
	
		// initialise our document structure
		InitDocumentData( &gDocument ) ;
		
		// try to read the file into the main display group
		gDocument.fModel = MyNewModelFromFile( &theFileSpec );
		if( gDocument.fModel )
		{
#if SHOW_BACKPLANE
			gDocument.fBackPlane = MyNewBackPlane( &gDocument );
			if( gDocument.fBackPlane )
			{		
#endif
				PositionCameras( &gDocument );
				MainEventLoop();
#if SHOW_BACKPLANE
			}
#endif
		}
				
		DisplayRelease();
		DisposeDocumentData( &gDocument ) ;
		
		//	Close our connection to the QuickDraw 3D library
		myStatus = Q3Exit();
		if ( myStatus == kQ3Failure )
		{
			DEBUGSTR("\pErExit returned failure.");
		}
	}	
}

//-------------------------------------------------------------------------------------------
//

void InitDocumentData( DocumentPtr theDocument ) 
{
	TQ3Point3D		myOrigin = { 0, 0, 0 } ;
	
	// sets up the 3d data for the scene
	//	Create view for QuickDraw 3D.
	theDocument->leftView = MyNewView( (GrafPtr)DisplayGetBuffer( kDSpBufferKind_LeftEye ) );
	theDocument->rightView = MyNewView( (GrafPtr)DisplayGetBuffer( kDSpBufferKind_RightEye ) );
	
	Q3View_GetCamera( theDocument->leftView, &theDocument->leftCamera );
	Q3View_GetCamera( theDocument->rightView, &theDocument->rightCamera );
	
	theDocument->centerCamera = MyNewCamera( (GrafPtr)DisplayGetBuffer( kDSpBufferKind_LeftEye ) );
	theDocument->interaxialHalfDistance = .5;
	
	// the display groups
	theDocument->fModel = NULL;
	theDocument->fBackPlane = NULL;
	
	// the drawing styles:
	theDocument->fInterpolation = Q3InterpolationStyle_New(kQ3InterpolationStyleNone) ;
	theDocument->fBackFacing = Q3BackfacingStyle_New(kQ3BackfacingStyleRemove ) ;
	theDocument->fFillStyle = Q3FillStyle_New(kQ3FillStyleFilled ) ;

	theDocument->fGroupScale = 1;				
	theDocument->fGroupCenter = myOrigin ;			

	// set the rotation matrix the identity matrix
	Q3Matrix4x4_SetIdentity(&theDocument->fRotation);	
					
	return ;
	
bail:
	// either we failed getting the model or the view
	// so we want to quit here
	ExitToShell() ;
	
}

void DisposeDocumentData( DocumentPtr theDocument)
{
	Q3Object_Dispose(theDocument->leftView) ;				// the view for the scene
	Q3Object_Dispose(theDocument->rightView) ;				// the view for the scene

	Q3Object_Dispose(theDocument->leftCamera) ;
	Q3Object_Dispose(theDocument->rightCamera) ;
	Q3Object_Dispose(theDocument->centerCamera) ;
	
	Q3Object_Dispose(theDocument->fModel) ;				// object in the scene being modelled
#if SHOW_BACKPLANE
	Q3Object_Dispose(theDocument->fBackPlane);			// stationary back plane
	Q3Object_Dispose( gBackPlaneTransform );
	Q3Object_Dispose( gMemoryStorage );
#endif
	Q3Object_Dispose(theDocument->fInterpolation) ;		// interpolation style used when rendering
	Q3Object_Dispose(theDocument->fBackFacing) ;		// whether to draw shapes that face away from the camera
	Q3Object_Dispose(theDocument->fFillStyle) ;			// whether drawn as solid filled object or decomposed to components
}
//-----------------------------------------------------------------------------
// assumes the port is set up before being called

TQ3Status DocumentDraw3DData( DocumentPtr theDocument )
{
	TQ3Status theStatus ;	

	SetPort( (GrafPtr)DisplayGetBuffer( kDSpBufferKind_LeftEye ) );
	Q3View_StartRendering( theDocument->leftView );
	do {
		theStatus = SubmitScene( theDocument, true, true ) ;
	} while( Q3View_EndRendering( theDocument->leftView ) == kQ3ViewStatusRetraverse );
	Q3View_Sync( theDocument->leftView );

	SetPort( (GrafPtr)DisplayGetBuffer( kDSpBufferKind_RightEye ) );
	Q3View_StartRendering( theDocument->rightView ) ;
	do {
		theStatus = SubmitScene( theDocument, false, true ) ;
	} while( Q3View_EndRendering( theDocument->rightView ) == kQ3ViewStatusRetraverse );
	Q3View_Sync( theDocument->rightView );
	
	DisplayShowBuffer();
	
	return theStatus ;	
}


//----------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------
//

short HiWrd(long aLong)
{
	return	(((aLong) >> 16) & 0xFFFF) ;
}

//-------------------------------------------------------------------------------------------
//

short LoWrd(long aLong)
{
	return	((aLong) & 0xFFFF) ;

}

//-------------------------------------------------------------------------------------------
//

void InitToolbox()
{
	Handle		menuBar = nil;

	MaxApplZone() ;
	MoreMasters() ; MoreMasters() ; MoreMasters() ; 
	
	InitGraf( &qd.thePort );
	InitFonts();
	InitWindows();

	FlushEvents( everyEvent, 0 ) ;
	// initialize application globals
	
	gQuitFlag = false;
	InitCursor();
	
}


//-------------------------------------------------------------------------------------------
//
void MainEventLoop()
{
	TQ3Matrix4x4	tmp;
	static float theAngle = 0;
	
	while( !Button() )
	{
		Q3Matrix4x4_SetRotate_XYZ(&tmp, 0.1, 0.12, 0.08);
		Q3Matrix4x4_Multiply(&gDocument.fRotation, &tmp, &gDocument.fRotation);

		theAngle -= .017;
#if SHOW_BACKPLANE
		Q3RotateTransform_SetAngle( gBackPlaneTransform, theAngle);
#endif

		DocumentDraw3DData( &gDocument ) ;
	}
}


//-------------------------------------------------------------------------------------------
//
void HandleKeyPress(EventRecord * /* event */ )
{}

//-------------------------------------------------------------------------------------------
//



