/*
********************************************************************************
**
** File: i*Glasses.c
**
** Description:
**
**	GoggleSprocket driver for the Virtual i/O i*Glasses.
**
********************************************************************************
*/
#include <Types.h>
#include <Files.h>

#include "GoggleSprocketDevices.h"

/*
********************************************************************************
** constants
********************************************************************************
*/
#define kDialogID				128
#define kItemOK					1

/*
********************************************************************************
** globals
********************************************************************************
*/
/* this device only supports a single "mode" */
GSpDeviceModeInfo gModeInfo = {
	/* mode format is normal */
	kGSpModeDataFormat_Interleaved,
	
	/* special attributes */
	kGSpModeAttribute_FullDisplayRequired,
	
	/* display id (none required) */
	0,
	
	/* display requirement ranges */
	{ 640, 640 },					/* width					*/
	{ 480, 480 },					/* height					*/
	{ 0, 0 },						/* depth					*/
	{ (60<<16), (60<<16) }			/* frequency (Fixed point)	*/
};

/*
********************************************************************************
**
** Name: GSpDevice_Open
**
** Description:
**
**	Called to allow the driver to do any setup.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_Open( void )
{
	return noErr;
}

/*
********************************************************************************
**
** Name: GSpDevice_Close
**
** Description:
**
**	Called to allow the driver to clean up.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_Close( void )
{
	return noErr;
}

/*
********************************************************************************
**
** Name: GSpDevice_Configure
**
** Description:
**
**	Called to tell the driver to display and manage its configuration dialog.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_Configure(
	Point *inUpperLeft				/* upper left corner of window, or NULL	*/
)
{
	DialogPtr theDialog;
	GrafPtr theSavePort;
	
	GetPort( &theSavePort );
	
	theDialog = GetNewDialog( kDialogID, NULL, (GrafPtr)-1 );
	if( theDialog )
	{	
		short theItem = 0;
		
		SetDialogDefaultItem( theDialog, kItemOK );
		
		// if an upper left is provided, use it (parameter may be NULL)
		if( inUpperLeft )
			MoveWindow( theDialog, inUpperLeft->h, inUpperLeft->v, true );
		ShowWindow( theDialog );
		
		// show the dialog
		while( theItem != kItemOK )
			ModalDialog( nil, &theItem );
			
		// release the dialog memory
		DisposeDialog( theDialog );
	}
	
	SetPort( theSavePort );
	
	return noErr;
}

/*
********************************************************************************
**
** Name: GSpDevice_GetFirstModeInfo
**
** Description:
**
**	Return the first available mode the device supports.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_GetFirstModeInfo(
	GSpDeviceModeInfo *outFirstModeInfo		/* mode information (output)	*/
)
{
	*outFirstModeInfo = gModeInfo;
	return noErr;
}

/*
********************************************************************************
**
** Name: GSpDevice_Open
**
** Description:
**
**	The driver open call.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_GetNextModeInfo(
	GSpDeviceModeInfo *inCurrentModeInfo,
	GSpDeviceModeInfo *outNextModeInfo
)
{
	return kGSpDeviceError_NoMoreModes;
}

/*
********************************************************************************
**
** Name: GSpDevice_SetVisibleEye
**
** Description:
**
**	Set the eye that can currently see the stereo image.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_SetVisibleEye(
	Boolean inLeftEyeVisible		/* TRUE = make left eye visible			*/
)
{
	return kGSpDeviceError_UnsupportedCall;
}

/*
********************************************************************************
**
** Name: GSpDevice_GetVisibleEye
**
** Description:
**
**	The driver open call.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_GetVisibleEye(
	Boolean *outLeftEyeVisible		/* TRUE = left eye is visible			*/
)
{
	return kGSpDeviceError_UnsupportedCall;
}

/*
********************************************************************************
**
** Name: GSpDevice_CustomBufferProcess
**
** Description:
**
**	The catch-all call that allows drivers with unique requirements to
**	operate.
**
********************************************************************************
*/
OSStatus GSpDevice_CustomBufferProcess(
			CGrafPtr inLeftEyeBuffer,		/* left eye image				*/
			CGrafPtr inRightEyeBuffer,		/* right eye image				*/
			CGrafPtr inDestBuffer,			/* dest eye image				*/
			Boolean inBuildLeftEyeBuffer,	/* 0 = build right eye image	*/
			CTabHandle ioColorTable,		/* src color table to use		*/
			Boolean *outShowDisplayBuffer,	/* 0 = driver will show output	*/
			Boolean *outCTabChanged )		/* 0 = driver didnt change ctab	*/
{
	return noErr;
}

/*
********************************************************************************
**
** Name: GSpDevice_GetDeviceKind
**
** Description:
**
**	Return the kind of device that this is.
**
********************************************************************************
*/
OSStatus							/* result code							*/
GSpDevice_GetDeviceKind(
	GSpDeviceKind *outDeviceKind
)
{
	*outDeviceKind = kGSpDeviceKind_Stereoscopic;
	return noErr;
}
