/*
********************************************************************************
**
** File: GSpTest_Aevt.cp
**
** Description:
**
**	Apple Event routines.
**
********************************************************************************
*/
#include "GSpTest_AEvt.h"
#include "GSpTest_EventLoop.h"

/*
********************************************************************************
** local prototypes
********************************************************************************
*/
static pascal OSErr DoAEOpenApp( AppleEvent, AppleEvent, long );
static pascal OSErr DoAEQuitApp( AppleEvent, AppleEvent, long );
static pascal OSErr DoAEOpenDoc( AppleEvent, AppleEvent, long );
static pascal OSErr DoAEPrintDoc( AppleEvent, AppleEvent, long );

/*
********************************************************************************
** static globals
********************************************************************************
*/
static AEEventHandlerUPP gAEOpenAppHandler, gAEQuitHandler, gAEOpenDocHandler,
	gAEPrintDocHandler;
	
/*
********************************************************************************
**
** Name: AEStartup
**
** Description:
**
**	Install core apple event handlers.
**
********************************************************************************
*/
OSErr			// result code
AEStartup( void )
{	
	OSErr theError = noErr;

	// release previous handlers
	AEShutdown();
	
	// allocate UPPs
	gAEOpenAppHandler = NewAEEventHandlerProc( DoAEOpenApp );
	gAEQuitHandler = NewAEEventHandlerProc(DoAEQuitApp);
	gAEOpenDocHandler = NewAEEventHandlerProc(DoAEOpenDoc);
	gAEPrintDocHandler = NewAEEventHandlerProc(DoAEPrintDoc);
	if( ( NULL == gAEOpenAppHandler ) ||
		( NULL == gAEQuitHandler ) ||
		( NULL == gAEOpenDocHandler ) ||
		( NULL == gAEPrintDocHandler ) )
	{
		theError = memFullErr;
	}
	
	//	install auto Open App
	if( noErr == theError )
		theError = AEInstallEventHandler( kCoreEventClass, kAEOpenApplication,
			gAEOpenAppHandler, 0L, false );
			
	// install auto Quit App								
	if( noErr == theError )
		theError = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, 
			gAEQuitHandler, 0L, false );
		
	// install auto Open Document
	if( noErr == theError )
		theError = AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments,
			gAEOpenDocHandler, 0L, false );
														
	// install auto Print Document
	if( noErr == theError )
		theError = AEInstallEventHandler( kCoreEventClass, kAEPrintDocuments,
			gAEPrintDocHandler, 0L, false );

	// handle error by removing AE handlers and releasing UPPs
	if( noErr != theError )
		AEShutdown();
		
	// done								
	return theError;
}

/*
********************************************************************************
**
** Name: AEShutdown
**
** Description:
**
**	Remove and deallocate core apple event handlers.
**
********************************************************************************
*/
void
AEShutdown( void )
{
	// remove open application handler
	if( gAEOpenAppHandler )
	{
		AERemoveEventHandler( kCoreEventClass, kAEOpenApplication,
			gAEOpenAppHandler, false );

		DisposeRoutineDescriptor( gAEOpenAppHandler );
		
		gAEOpenAppHandler = NULL;
	}
	
	// remove quit application handler
	if( gAEQuitHandler )
	{
		AERemoveEventHandler( kCoreEventClass, kAEQuitApplication,
			gAEQuitHandler, false );

		DisposeRoutineDescriptor( gAEQuitHandler );
		
		gAEQuitHandler = NULL;
	}
	
	// remove open document handler
	if( gAEOpenDocHandler )
	{
		AERemoveEventHandler( kCoreEventClass, kAEOpenDocuments,
			gAEOpenDocHandler, false );

		DisposeRoutineDescriptor( gAEOpenDocHandler );
		
		gAEOpenDocHandler = NULL;
	}
	
	// remove print document handler
	if( gAEPrintDocHandler )
	{
		AERemoveEventHandler( kCoreEventClass, kAEPrintDocuments,
			gAEPrintDocHandler, false );

		DisposeRoutineDescriptor( gAEPrintDocHandler );
		
		gAEPrintDocHandler = NULL;
	}
}

/*
********************************************************************************
**
** Name: DoAEOpenApp
**
** Description:
**
**	Apple Event open application handler.
**
********************************************************************************
*/
static pascal OSErr			// result code
DoAEOpenApp( 
	AppleEvent inEvent,		// input event
	AppleEvent ioReply,		// event reply
	long inRefCon			// reference constant
)
{
	#pragma unused( inEvent, ioReply, inRefCon )

	return noErr;
}

/*
********************************************************************************
**
** Name: DoAEQuitApp
**
** Description:
**
**	Apple Event quit application handler.
**
********************************************************************************
*/
static pascal OSErr			// result code
DoAEQuitApp( 
	AppleEvent inEvent,		// input event
	AppleEvent ioReply,		// event reply
	long inRefCon			// reference constant
)
{
	#pragma unused( inEvent, ioReply, inRefCon )

	// set the global quit flag
	gDoneFlag = true;
	
	return noErr;
}

/*
********************************************************************************
**
** Name: DoAEOpenDoc
**
** Description:
**
**	Apple Event open document handler.
**
********************************************************************************
*/
static pascal OSErr			// result code
DoAEOpenDoc( 
	AppleEvent inEvent,		// input event
	AppleEvent ioReply,		// event reply
	long inRefCon			// reference constant
)
{
	#pragma unused( inEvent, ioReply, inRefCon )

	return noErr;
}
										
/*
********************************************************************************
**
** Name: DoAEPrintDoc
**
** Description:
**
**	Apple Event print document handler.
**
********************************************************************************
*/
static pascal OSErr			// result code
DoAEPrintDoc( 
	AppleEvent inEvent,		// input event
	AppleEvent ioReply,		// event reply
	long inRefCon			// reference constant
)
{
	#pragma unused( inEvent, ioReply, inRefCon )

	return noErr;
}
