/*
	File:		MouseModule.c

	Contains:	HID Module for USB Mouse

	Version:	xxx put version here xxx

	Copyright:	 1997-1998 by Apple Computer, Inc., all rights reserved.

*/

#include <Types.h>
#include <Devices.h>
#include <processes.h>
#include <DriverServices.h>
#include <USB.h>

#include "MouseModule.h"

usbMousePBStruct myMousePB;

void InitParamBlock(USBDeviceRef theDeviceRef, USBPB * paramblock)
{
	paramblock->usbReference = theDeviceRef;
	paramblock->pbVersion = kUSBCurrentPBVersion;
	paramblock->usbWIndex = 0; 			
	paramblock->usbBuffer = nil;		
	paramblock->usbStatus = noErr;
	paramblock->usbReqCount = 0;
	paramblock->usbWValue = 0;
	paramblock->usbFlags = 0;
}



Boolean immediateError(OSStatus err)
{
	return((err != kUSBPending) && (err != noErr) );
}

void MouseModuleInitiateTransaction(USBPB *pb)
{
register usbMousePBStruct *pMousePB;
OSStatus myErr;

	pMousePB = (usbMousePBStruct *)(pb);
	pMousePB->transDepth++;
	if (pMousePB->transDepth < 0)
	{
		USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: transDepth < 0 (initiation)", pMousePB->pb.usbRefcon );
	}
	
	if (pMousePB->transDepth > 1)
	{
		USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: transDepth > 1 (initiation)", pMousePB->pb.usbRefcon );
	}
	
	switch(pMousePB->pb.usbRefcon & ~kRetryTransaction)
	{
		case kGetFullConfiguration:
			InitParamBlock(pMousePB->deviceRef, &pMousePB->pb);
			
			pMousePB->pb.usbRefcon |= kCompletionPending;
			pMousePB->pb.usbCompletion = (USBCompletion)TransactionCompletionProc;
			myErr = USBGetFullConfigurationDescriptor(pb);
			if(immediateError(myErr))
			{
				USBExpertFatalError(pMousePB->pb.usbReference, kUSBInternalErr, "\pUSBHIDMouseModule: kGetFullConfiguration (ImmediateError)", myErr);
			}
			break;
			
		case kFindInterfaceAndSetProtocol:
			myErr = FindHIDInterfaceByNumber(pMousePB->pFullConfigDescriptor, pMousePB->interfaceDescriptor.interfaceNumber, &pMousePB->pInterfaceDescriptor );	// find the HID interface
			if ((pMousePB->pInterfaceDescriptor == NULL) || (myErr != noErr))
			{
				USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: Interface not found", myErr);
				pMousePB->pb.usbRefcon = kReturnFromDriver;
			}
			
			InitParamBlock(pMousePB->deviceRef, &pMousePB->pb);
			
			pMousePB->pb.usbBMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);			
			pMousePB->pb.usbBRequest = kHIDRqSetProtocol;
			pMousePB->pb.usbWValue = kHIDBootProtocolValue; 
			pMousePB->pb.usbWIndex = pMousePB->pInterfaceDescriptor->interfaceNumber;
			pMousePB->pb.usbCompletion = (USBCompletion)TransactionCompletionProc;
			
			pMousePB->pb.usbRefcon |= kCompletionPending;
			myErr = USBDeviceRequest(&pMousePB->pb);
			if (immediateError(myErr))
			{
				USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: kSetProtocol (ImmediateError)", myErr);
			}
			break;
			
		case kSetIdleRequest:
			InitParamBlock(pMousePB->deviceRef, &pMousePB->pb);
			
			pMousePB->pb.usbBMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);			
			
			pMousePB->pb.usbBRequest = kHIDRqSetIdle;
			if ((myMousePB.deviceDescriptor.vendor == USB_CONSTANT16(0x1452)) &&
				(myMousePB.deviceDescriptor.product == USB_CONSTANT16(0x0301)) &&
				(pMousePB->deviceDescriptor.devRel == USB_CONSTANT16(0x0004)))
			{
				pMousePB->pb.usbWValue = ((50/4)<<8); 				// force a read completion if idle for more than 50ms
			}
			else
			{
				pMousePB->pb.usbWValue = 0; 						// Turn off SetIdle time (set it to 0ms)
			}
			pMousePB->pb.usbWIndex = pMousePB->interfaceDescriptor.interfaceNumber;
			pMousePB->pb.usbCompletion = (USBCompletion)TransactionCompletionProc;
			
			pMousePB->pb.usbRefcon |= kCompletionPending;

			myErr = USBDeviceRequest(&pMousePB->pb);
			if(immediateError(myErr))
			{
				USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: kSetIdleRequest - immediate error", myErr);
			}
			break;

		case kFindAndOpenInterruptPipe:
			pMousePB->pb.usbClassType = kUSBInterrupt;
			pMousePB->pb.usbOther = 0;			
			pMousePB->pb.usbBMRequestType = USBMakeBMRequestType(kUSBIn, kUSBClass, kUSBInterface);
			pMousePB->pb.usbFlags = kUSBIn;
			pMousePB->pb.usbBuffer = pMousePB->pInterfaceDescriptor;
			pMousePB->pb.usbReqCount = (UInt8*)pMousePB->pInterfaceDescriptor - (UInt8*)pMousePB->pFullConfigDescriptor;
			myErr = USBFindNextEndpointDescriptorImmediate( &pMousePB->pb );
			if((immediateError(myErr)) || (pMousePB->pb.usbBuffer == nil))
			{
				USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: Endpoint not found", myErr);
				pMousePB->pb.usbRefcon = kReturnFromDriver;
			}
			else
			{
				pMousePB->pEndpointDescriptor = (USBEndPointDescriptorPtr) pMousePB->pb.usbBuffer;

				InitParamBlock(pMousePB->deviceRef, &pMousePB->pb);
				
				pMousePB->pb.usbFlags = kUSBIn;
				pMousePB->pb.usbWValue = USBToHostWord(pMousePB->pEndpointDescriptor->maxPacketSize);
				pMousePB->pb.usbClassType = kUSBInterrupt;
				pMousePB->pb.usbOther = (pMousePB->pEndpointDescriptor->endpointAddress & 0x0f);
				pMousePB->pb.usbRefcon |= kCompletionPending;
				pMousePB->pb.usbCompletion = (USBCompletion)TransactionCompletionProc;
		
				myErr = USBOpenPipe( &pMousePB->pb );
				if(immediateError(myErr))
				{
					USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: USBOpenPipe Failed (ImmediateError)", myErr);
				}
			}
			break;
		
		case kReadInterruptPipe:
			InitParamBlock(pMousePB->pipeRef, &pMousePB->pb);

			pMousePB->pb.usbBuffer = (Ptr)pMousePB->hidReport;
			pMousePB->pb.usbReqCount = USBToHostWord(pMousePB->pEndpointDescriptor->maxPacketSize);
			pMousePB->pb.usbWIndex = pMousePB->interfaceDescriptor.interfaceNumber;	
			pMousePB->pb.usbCompletion = (USBCompletion)TransactionCompletionProc;
			
			pMousePB->pb.usbRefcon |= kCompletionPending;
		
			myErr = USBIntRead(&pMousePB->pb);
			if(immediateError(myErr))
			{
				USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: Read Interrupt Pipe (ImmediateError)", myErr);
			}
			break;
			
		default:
			USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: Transaction initiated with bad refcon value", pMousePB->pb.usbRefcon);
			pMousePB->pb.usbRefcon = kUndefined + kReturnFromDriver;
			break;
	}
	
// At this point the control is returned to the system.  If a USB transaction
// has been initiated, then it will call the Complete procs
// (below) to handle the results of the transaction.
}


void TransactionCompletionProc(USBPB *pb)
{
register usbMousePBStruct *pMousePB;
unsigned char	* errstring;
USBPipeState 	pipeState;

	pMousePB = (usbMousePBStruct *)(pb);
	pMousePB->transDepth--;
	if (pMousePB->transDepth < 0)
	{
		USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: transDepth < 0 (completion)", pMousePB->pb.usbRefcon );
	}
	
	if (pMousePB->transDepth > 1)
	{
		USBExpertFatalError(pMousePB->deviceRef, kUSBInternalErr, "\pUSBHIDMouseModule: transDepth > 1 (completion)", pMousePB->pb.usbRefcon );
	}
	
	if(pMousePB->pb.usbStatus != noErr)														// was there an error?
	{
		switch(pMousePB->pb.usbRefcon & 0x0fff)												// yes, so show where the error occurred
		{
			case kGetFullConfiguration:  		errstring = "\pUSBHIDMouseModule: Error during GetFullConfiguration"; break;
			case kFindInterfaceAndSetProtocol:	errstring = "\pUSBHIDMouseModule: Error during FindInterfaceAndSetProtocol"; break;
			case kSetIdleRequest:				errstring = "\pUSBHIDMouseModule: Error during kSetIdleRequest"; break;
			case kFindAndOpenInterruptPipe:  	errstring = "\pUSBHIDMouseModule: Error during FindAndOpenInterruptPipe"; break;
			case kReadInterruptPipe: 	 		errstring = "\pUSBHIDMouseModule: Error during ReadInterruptPipe"; break;
			default: 	 						errstring = "\pUSBHIDMouseModule: Error occurred, but state is unknown"; break;
		};
		USBExpertFatalError(pMousePB->deviceRef, pMousePB->pb.usbStatus, errstring, (pMousePB->pb.usbRefcon & 0x0fff));
		
		pMousePB->pb.usbRefcon &= ~(kCompletionPending + kReturnFromDriver);				// set up to retry the transaction
		pMousePB->pb.usbRefcon |= kRetryTransaction;
		pMousePB->retryCount--;
		
		if ((pMousePB->retryCount == 1) && ((pMousePB->pb.usbRefcon&pMousePB->pb.usbRefcon & 0x0fff) == kSetIdleRequest))
		{
			USBExpertStatus(pMousePB->deviceRef, "\pUSBHIDMouseModule: Device doesn't accept SetIdle", pMousePB->deviceRef);
			pMousePB->pb.usbRefcon = kFindAndOpenInterruptPipe;
			pMousePB->pb.usbStatus = noErr;
		}
		else
		{
			if ((!pMousePB->retryCount)	|| (pMousePB->pb.usbStatus == kUSBAbortedError))	// have we exhausted the retries?
			{																				// or received an abort?
				USBExpertStatus(pMousePB->deviceRef, "\pUSBHIDMouseModule: Pipe abort or unable to recover from error", pMousePB->deviceRef);
				pMousePB->pb.usbRefcon = kReturnFromDriver;									// if so, just exit.
			}
			else																			// if it didn't abort and there's retries left, then...
			{
				if (pMousePB->pipeRef)														// check if the pipe is open.
				{
					USBGetPipeStatusByReference(pMousePB->pipeRef, &pipeState);				// yes, so what it's state?
					if (pipeState != kUSBActive)											// if it's not active, try to clear it.  It might be stalled...
					{
						USBExpertStatus(pMousePB->deviceRef, "\pUSBHIDMouseModule: Pipe is open and stalled, clearing stall...", pMousePB->deviceRef);
						USBClearPipeStallByReference(pMousePB->pipeRef);
					}
				}
			}
		}
	}
	else
	{
		pMousePB->pb.usbRefcon &= ~kRetryTransaction;
		pMousePB->retryCount = kMouseRetryCount;
	}

	if (pMousePB->pb.usbRefcon & kCompletionPending)			 
	{												
		pMousePB->pb.usbRefcon &= ~(kCompletionPending + kReturnFromDriver);
		switch(pMousePB->pb.usbRefcon)
		{
			case kGetFullConfiguration:
				pMousePB->pFullConfigDescriptor = pMousePB->pb.usbBuffer;
				if (pMousePB->pFullConfigDescriptor == nil)
				{
					USBExpertFatalError(pMousePB->pb.usbReference, kUSBInternalErr, "\pUSBHIDMouseModule: USBGetFullConfiguration - pointer is nil", pMousePB->pb.usbRefcon);
					pMousePB->pb.usbRefcon = kReturnFromDriver;
				}
				else
				{
					pMousePB->pb.usbRefcon = kFindInterfaceAndSetProtocol;
				}
				break;
				
			case kFindInterfaceAndSetProtocol:
				pMousePB->pb.usbRefcon = kSetIdleRequest;
				break;
				
			case kSetIdleRequest:
				pMousePB->pb.usbRefcon = kFindAndOpenInterruptPipe;
				break;
				
			case kFindAndOpenInterruptPipe:
				pMousePB->pipeRef = pMousePB->pb.usbReference;
				pMousePB->pb.usbRefcon = kReadInterruptPipe;
				break;
				
			case kReadInterruptPipe:
				NotifyRegisteredHIDUser(pMousePB->hidDeviceType, pMousePB->hidReport);
				pMousePB->pb.usbRefcon = kReadInterruptPipe;
				break;

		}
	}
	if (!(pMousePB->pb.usbRefcon & kReturnFromDriver))
		MouseModuleInitiateTransaction(pb);
}


void InterfaceEntry(UInt32 interfacenum, USBInterfaceDescriptorPtr pInterfaceDescriptor, USBDeviceDescriptorPtr pDeviceDescriptor, USBDeviceRef device)
{
#pragma unused (interfacenum)

static Boolean beenThereDoneThat = false;

	if(beenThereDoneThat)
	{
		USBExpertFatalError(device, kUSBInternalErr, "\pUSBHIDMouseModule is not reentrant", 0);
		return;
	}
	beenThereDoneThat = true;
	
//	DebugStr("\pIn Mouse Interface Entry routine");

	myMousePB.deviceDescriptor = *pDeviceDescriptor;				
	myMousePB.interfaceDescriptor = *pInterfaceDescriptor;				
	myMousePB.transDepth = 0;							
	myMousePB.retryCount = kMouseRetryCount;
	  
	myMousePB.pSHIMInterruptRoutine = nil;
	myMousePB.pSavedInterruptRoutine = nil;

	myMousePB.deviceRef = device;		
	myMousePB.interfaceRef = nil;		
	myMousePB.pipeRef = nil;		
	
	InitParamBlock(device, &myMousePB.pb);
	
	myMousePB.pb.usbReference = device;
	myMousePB.pb.pbLength = sizeof(usbMousePBStruct);
	myMousePB.pb.usbRefcon = kGetFullConfiguration;		
	
	if ((myMousePB.deviceDescriptor.vendor == USB_CONSTANT16(0x046e)) &&
		(myMousePB.deviceDescriptor.product == USB_CONSTANT16(0x6782)))
	{
		myMousePB.unitsPerInch = (Fixed)(100<<16);
	}
	else
	{
		myMousePB.unitsPerInch = (Fixed)(400<<16);
	}
	
	myMousePB.pCursorDeviceInfo = 0;				
	USBHIDControlDevice(kHIDEnableDemoMode,0);

	MouseModuleInitiateTransaction(&myMousePB.pb);
}



